﻿using System;
using System.Configuration;
using System.Windows.Forms;
using iTextSharp;
using iTextSharp.text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Collections;
using System.Data.Odbc;
using System.Data;

namespace AjpdSoftIndexarTextoFicherosPDF
{
    class Utilidades
    {

        //para tipo de DNS (ODBC)
        public enum DataSourceType { System, User }

        //para la conexión con la BD ODBC 
        //para añadir el texto indexado de los PDF
        public OdbcConnection conexionBDODBC = new OdbcConnection();


        //leer valor del parámetro de configuración en el fichero .config
        public string leerValorConfiguracion (string clave)
        {
            try
            {
                string resultado = 
                    ConfigurationManager.AppSettings[clave].ToString();
                return resultado;
            }
            catch
            {
                return "";
            } 
        }

        //escribir valor de parámetro de configuración en fichero .config
        public void guardarValorConfiguracion(string clave, string valor)
        {
            try
            {
                //La línea siguiente no funcionará bien en tiempo de diseño
                //pues VC# usa el fichero xxx.vshost.config en la depuración
                //Configuration config = 
                //    ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
                //sí pues la cambiamos por:
                Configuration ficheroConfXML = 
                    ConfigurationManager.OpenExeConfiguration(Application.ExecutablePath);

                //eliminamos la clave actual (si existe), si no la eliminamos
                //los valores se irán acumulando separados por coma
                ficheroConfXML.AppSettings.Settings.Remove(clave);

                //asignamos el valor en la clave indicada
                ficheroConfXML.AppSettings.Settings.Add(clave, valor);

                //guardamos los cambios definitivamente en el fichero de configuración
                ficheroConfXML.Save(ConfigurationSaveMode.Modified);
            }
            catch
            {
               /* MessageBox.Show("Error al guardar valor de configuración: " +
                    System.Environment.NewLine + System.Environment.NewLine +
                    ex.GetType().ToString() + System.Environment.NewLine +
                    ex.Message, "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);*/
            } 
        }

        //obtiene los metadatos del fichero PDF indicado
        public string obtenerMetaDatosFicheroPDF(string ficheroPDFIndexar)
        {
            string mMetadatos = "";            
            try
            {
                PdfReader ficheroPDF = new PdfReader(ficheroPDFIndexar);
                Dictionary<string, string> metadatosPDF = ficheroPDF.Info;
                foreach (KeyValuePair<string, string> clavesMetadatosPDF in metadatosPDF)
                {
                    if (mMetadatos != "")
                    {
                        mMetadatos = mMetadatos + Environment.NewLine +
                            (clavesMetadatosPDF.Key + " ===> " + clavesMetadatosPDF.Value);
                    }
                    else
                    {
                        mMetadatos = (clavesMetadatosPDF.Key + " ===> " +
                            clavesMetadatosPDF.Value);
                    }
                }
                return mMetadatos;
            }
            catch
            {
                return null;
            }
        }

        //obtiene el texto del fichero PDF indicado
        public string obtenerTextoFicheroPDF(string ficheroPDF)
        {
            StringBuilder textoPDFIndexado = new StringBuilder();
            try
            {
                if (File.Exists(ficheroPDF))
                {
                    PdfReader pdfReader = new PdfReader(ficheroPDF);

                    for (int page = 1; page <= pdfReader.NumberOfPages; page++)
                    {
                        ITextExtractionStrategy strategy =
                            new SimpleTextExtractionStrategy();
                        string currentText =
                            PdfTextExtractor.GetTextFromPage(pdfReader, page, strategy);

                        currentText = Encoding.UTF8.GetString(ASCIIEncoding.Convert(
                            Encoding.Default, Encoding.UTF8,
                            Encoding.Default.GetBytes(currentText)));
                        textoPDFIndexado.Append(currentText);
                        pdfReader.Close();
                    }
                }
                return textoPDFIndexado.ToString();
            }
            catch
            {
                return null;
            }
        }


        //obtiene todos los data sources (orígenes de datos) del sistema
        public System.Collections.SortedList obtenerDataSourcesSistema()
        {
            System.Collections.SortedList listaODBC = new System.Collections.SortedList();

            Microsoft.Win32.RegistryKey reg = 
                (Microsoft.Win32.Registry.LocalMachine).OpenSubKey("Software");
            if (reg != null)
            {
                reg = reg.OpenSubKey("ODBC");
                if (reg != null)
                {
                    reg = reg.OpenSubKey("ODBC.INI");
                    if (reg != null)
                    {
                        reg = reg.OpenSubKey("ODBC Data Sources");
                        if (reg != null)
                        {
                            // Obtener todas las entradas DSN definidas en DSN_LOC_IN_REGISTRY.
                            foreach (string sName in reg.GetValueNames())
                            {
                                listaODBC.Add(sName, DataSourceType.System);
                            }
                        }
                        try
                        {
                            reg.Close();
                        }
                        catch { /* ignorar un posible error */ }
                    }
                }
            }

            return listaODBC;
        }

        //obtiene todos los data sources (orígenes de datos) del usuario
        public SortedList obtenerDataSourcesUsuario()
        {
            SortedList listaODBC = new SortedList();

            Microsoft.Win32.RegistryKey reg =
                (Microsoft.Win32.Registry.CurrentUser).OpenSubKey("Software");
            if (reg != null)
            {
                reg = reg.OpenSubKey("ODBC");
                if (reg != null)
                {
                    reg = reg.OpenSubKey("ODBC.INI");
                    if (reg != null)
                    {
                        reg = reg.OpenSubKey("ODBC Data Sources");
                        if (reg != null)
                        {
                            // Obtener todas las entradas DSN definidas en DSN_LOC_IN_REGISTRY.
                            foreach (string sName in reg.GetValueNames())
                            {
                                listaODBC.Add(sName, DataSourceType.User);
                            }
                        }
                        try
                        {
                            reg.Close();
                        }
                        catch { /* ignorar un posible error */ }
                    }
                }
            }

            return listaODBC;
        }

        public bool existeRegistro(string ficheroPDF, 
            string campoFicheroPDF, string nombreTabla)
        {
            try
            {
                OdbcCommand comandoSQL = conexionBDODBC.CreateCommand();
                comandoSQL.CommandText = "select count(*) from " + 
                    nombreTabla + " where " + campoFicheroPDF + " = ?";

                OdbcParameter parametroSQL = new OdbcParameter();
                parametroSQL.DbType = System.Data.DbType.String;
                parametroSQL.Value = ficheroPDF;
                comandoSQL.Parameters.Add(parametroSQL);

                OdbcDataReader resultadoSQL = 
                    comandoSQL.ExecuteReader(CommandBehavior.CloseConnection);
                resultadoSQL.Read();
                Int32 numero = resultadoSQL.GetInt32(0);
                //resultadoSQL.Close(); 
                return numero > 0;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error de base de datos: " +
                      System.Environment.NewLine + System.Environment.NewLine +
                      ex.GetType().ToString() + System.Environment.NewLine +
                      ex.Message, "Error al crear PDF",
                      MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }          
        }



        //Insertar registro en base de datos ODBC con los datos del PDF indexado
        public void insertarRegistroBD (string ficheroPDF, string textoPDF, 
            string metadatosPDF, string campoFichero, string campoTexto, 
            string campoMetaDatos, string nombreTabla, string campoFecha, 
            bool reemplazar)
        {
            try
            {
                bool insertar = true;

                if (reemplazar)
                {
                    insertar = !existeRegistro(ficheroPDF, campoFichero, nombreTabla);
                }

                //actualizar registro ya existente
                if (!insertar)
                {
                    OdbcCommand comandoSQL = conexionBDODBC.CreateCommand();
                    comandoSQL.CommandText = "update " + nombreTabla + " set " +
                        campoTexto + " = ?, " +
                        campoMetaDatos + " = ?, " +
                        campoFecha + " = ?" +
                        " where " + campoFichero + " = ?";

                    OdbcParameter parametroSQL = new OdbcParameter();
                    parametroSQL.DbType = System.Data.DbType.String;
                    parametroSQL.Value = textoPDF;
                    comandoSQL.Parameters.Add(parametroSQL);

                    parametroSQL = new OdbcParameter();
                    parametroSQL.DbType = System.Data.DbType.String;
                    parametroSQL.Value = metadatosPDF;
                    comandoSQL.Parameters.Add(parametroSQL);

                    parametroSQL = new OdbcParameter();
                    parametroSQL.DbType = System.Data.DbType.Date;
                    parametroSQL.Value = DateTime.Now;
                    comandoSQL.Parameters.Add(parametroSQL);

                    parametroSQL = new OdbcParameter();
                    parametroSQL.DbType = System.Data.DbType.String;
                    parametroSQL.Value = ficheroPDF;
                    comandoSQL.Parameters.Add(parametroSQL);

                    comandoSQL.ExecuteNonQuery();
                }

                //insertar nuevo registro
                if (insertar)
                {
                    OdbcCommand comandoSQL = conexionBDODBC.CreateCommand();
                    comandoSQL.CommandText = "insert into " + nombreTabla +
                        " (" + campoFichero + "," + campoTexto +
                        "," + campoMetaDatos + ", " + campoFecha +
                        ") values (?, ?, ?, ?)";

                    OdbcParameter parametroSQL = new OdbcParameter();
                    parametroSQL.DbType = System.Data.DbType.String;
                    parametroSQL.Value = ficheroPDF;
                    comandoSQL.Parameters.Add(parametroSQL);

                    parametroSQL = new OdbcParameter();
                    parametroSQL.DbType = System.Data.DbType.String;
                    parametroSQL.Value = textoPDF;
                    comandoSQL.Parameters.Add(parametroSQL);

                    parametroSQL = new OdbcParameter();
                    parametroSQL.DbType = System.Data.DbType.String;
                    parametroSQL.Value = metadatosPDF;
                    comandoSQL.Parameters.Add(parametroSQL);

                    parametroSQL = new OdbcParameter();
                    parametroSQL.DbType = System.Data.DbType.Date;
                    parametroSQL.Value = DateTime.Now;
                    comandoSQL.Parameters.Add(parametroSQL);

                    comandoSQL.ExecuteNonQuery();

                    /*  
                    //Segundo método de inserción SQL con parámetros                 
                    OdbcCommand cmd = new OdbcCommand();
                    cmd.Connection = conexionBDODBC;
                    cmd.CommandText = "INSERT INTO " + nombreTabla + " (" +
                        campoFichero + ", " + campoTexto + ", " +
                        campoMetaDatos + ", " + campoFecha +
                        " ) VALUES (@ficheroPDF, @textoPDF, @metadatosPDF, @fecha);";
                    cmd.Parameters.AddWithValue("@ficheroPDF", ficheroPDF);
                    cmd.Parameters.AddWithValue("@textoPDF", textoPDF);
                    cmd.Parameters.AddWithValue("@metadatosPDF", metadatosPDF);
                    cmd.Parameters.AddWithValue("@fecha", DateTime.Now);
                    cmd.ExecuteNonQuery();
                    */
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error de base de datos: " +
                      System.Environment.NewLine + System.Environment.NewLine +
                      ex.GetType().ToString() + System.Environment.NewLine +
                      ex.Message, "Error al crear PDF",
                      MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
